package com.yanzhu.flowable.service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yanzhu.common.core.domain.AjaxResult;
import com.yanzhu.project.domain.ProProjectApply;
import com.yanzhu.system.domain.FlowProcDefDto;
import com.yanzhu.system.domain.flowable.FlowQueryVo;
import org.flowable.bpmn.model.FlowElement;
import org.flowable.bpmn.model.UserTask;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

/**
 * @author Tony
 * @date 2021-04-03 14:41
 */
public interface IFlowDefinitionService {

    boolean exist(String processDefinitionKey);

    /**
     * 流程定义列表
     *
     * @param flowQueryVo  流程查询
     * @return 流程定义分页列表数据
     */
    Page<FlowProcDefDto> list(FlowQueryVo flowQueryVo);

    /**
     * 导入流程文件
     * 当每个key的流程第一次部署时，指定版本为1。对其后所有使用相同key的流程定义，
     * 部署时版本会在该key当前已部署的最高版本号基础上加1。key参数用于区分流程定义
     * @param name
     * @param category
     * @param in
     */
    void importFile(String name, String category, String OwnerDeptId, InputStream in);

    /**
     * 读取xml
     * @param deployId
     * @return
     */
    AjaxResult readXml(String deployId) throws IOException;

    /**
     * 读取流程节点
     * @param deployId
     * @return
     */
    public List<FlowElement> readNodes(String deployId);

    /**
     * 读取流程节点
     * @param deployId
     * @return
     */
    public List<UserTask> findFlowNodes(String deployId);

    /**
     * 根据流程定义ID启动流程实例
     *
     * @param procDefId 流程ID
     * @param proProjectApply 申请表单
     * @return
     */
    AjaxResult startProcessInstanceById(String procDefId, ProProjectApply proProjectApply);


    /**
     * 激活或挂起流程定义
     *
     * @param state    状态
     * @param deployId 流程部署ID
     */
    void updateState(Integer state, String deployId);


    /**
     * 删除流程定义
     *
     * @param deployId 流程部署ID act_ge_bytearray 表中 deployment_id值
     */
    void delete(String deployId);


    /**
     * 读取图片文件
     * @param deployId
     * @return
     */
    InputStream readImage(String deployId);
}
