package com.yanzhu.framework.web.service;

import com.yanzhu.common.core.domain.entity.SysDept;
import com.yanzhu.common.core.domain.entity.SysRole;
import com.yanzhu.common.core.text.Convert;
import com.yanzhu.system.service.ISysDeptService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import com.yanzhu.common.core.domain.entity.SysUser;
import com.yanzhu.common.core.domain.model.LoginUser;
import com.yanzhu.common.enums.UserStatus;
import com.yanzhu.common.exception.ServiceException;
import com.yanzhu.common.utils.MessageUtils;
import com.yanzhu.common.utils.StringUtils;
import com.yanzhu.system.service.ISysUserService;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 用户验证处理
 *
 * @author yanZhu
 */
@Service
public class UserDetailsServiceImpl implements UserDetailsService
{
    private static final Logger log = LoggerFactory.getLogger(UserDetailsServiceImpl.class);

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISysDeptService sysDeptService;

    @Autowired
    private SysPasswordService passwordService;

    @Autowired
    private SysPermissionService permissionService;

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException
    {
        SysUser user = userService.selectUserByUserName(username);
        if (StringUtils.isNull(user))
        {
            log.info("登录用户：{} 不存在.", username);
            throw new ServiceException(MessageUtils.message("user.not.exists"));
        }
        else if (UserStatus.DELETED.getCode().equals(user.getDelFlag()))
        {
            log.info("登录用户：{} 已被删除.", username);
            throw new ServiceException(MessageUtils.message("user.password.delete"));
        }
        else if (UserStatus.DISABLE.getCode().equals(user.getStatus()))
        {
            log.info("登录用户：{} 已被停用.", username);
            throw new ServiceException(MessageUtils.message("user.blocked"));
        }

        //设置项目单位信息
        if(!user.isAdmin()){
            String as = user.getDept().getAncestors()+","+user.getDeptId();
            //判断是否是段队管理员
            String[] ancestorsIds = as.split(",");
            List<Long> deptIds = Arrays.stream(ancestorsIds).map(ancestors -> Long.parseLong(ancestors)).collect(Collectors.toList());
            List<SysDept> depts = sysDeptService.selectParentDeptByIds(deptIds);
            if(StringUtils.isNotEmpty(depts)){
                user.setParDeptId(depts.get(0).getDeptId());
                user.setParDeptName(depts.get(0).getDeptName());
            }
        }else{
            user.setParDeptId(100L);
            user.setParDeptName("中铁建设集团有限公司");
        }
        passwordService.validate(user);
        return createLoginUser(user);
    }

    public UserDetails createLoginUser(SysUser user)
    {
        return new LoginUser(user.getUserId(), user.getDeptId(), user, permissionService.getMenuPermission(user));
    }
}
