package com.yanzhu.web;

import com.yanzhu.common.annotation.Log;
import com.yanzhu.common.core.controller.BaseController;
import com.yanzhu.common.core.domain.AjaxResult;
import com.yanzhu.common.core.page.TableDataInfo;
import com.yanzhu.common.enums.BusinessType;
import com.yanzhu.common.utils.poi.ExcelUtil;
import com.yanzhu.project.domain.ProProjectApplyDetailCheck;
import com.yanzhu.project.service.IProProjectApplyDetailCheckService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 申请详情验收Controller
 * 
 * @author yanZhu
 * @date 2024-09-01
 */
@RestController
@RequestMapping("/manage/proProjectApplyDetailCheck")
public class ProProjectApplyDetailCheckController extends BaseController
{
    @Autowired
    private IProProjectApplyDetailCheckService proProjectApplyDetailCheckService;

    /**
     * 查询申请详情验收列表
     */
    @PreAuthorize("@ss.hasPermi('manage:proProjectApplyDetailCheck:list')")
    @GetMapping("/list")
    public TableDataInfo list(ProProjectApplyDetailCheck proProjectApplyDetailCheck)
    {
        startPage();
        List<ProProjectApplyDetailCheck> list = proProjectApplyDetailCheckService.selectProProjectApplyDetailCheckList(proProjectApplyDetailCheck);
        return getDataTable(list);
    }

    /**
     * 导出申请详情验收列表
     */
    @PreAuthorize("@ss.hasPermi('manage:proProjectApplyDetailCheck:export')")
    @Log(title = "申请详情验收", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ProProjectApplyDetailCheck proProjectApplyDetailCheck)
    {
        List<ProProjectApplyDetailCheck> list = proProjectApplyDetailCheckService.selectProProjectApplyDetailCheckList(proProjectApplyDetailCheck);
        ExcelUtil<ProProjectApplyDetailCheck> util = new ExcelUtil<ProProjectApplyDetailCheck>(ProProjectApplyDetailCheck.class);
        util.exportExcel(response, list, "申请详情验收数据");
    }

    /**
     * 获取申请详情验收详细信息
     * @PreAuthorize("@ss.hasPermi('manage:proProjectApplyDetailCheck:query')")
     */
    @GetMapping(value = "/findAll/{detailId}")
    public AjaxResult findAll(@PathVariable("detailId") Long detailId)
    {
        return success(proProjectApplyDetailCheckService.selectProProjectApplyDetailCheckByDetailId(detailId));
    }

    /**
     * 新增申请详情验收
     * @PreAuthorize("@ss.hasPermi('manage:proProjectApplyDetailCheck:add')")
     */
    @Log(title = "申请详情验收", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody ProProjectApplyDetailCheck proProjectApplyDetailCheck)
    {
        return toAjax(proProjectApplyDetailCheckService.insertProProjectApplyDetailCheck(proProjectApplyDetailCheck));
    }

    /**
     * 修改申请详情验收
     */
    @PreAuthorize("@ss.hasPermi('manage:proProjectApplyDetailCheck:edit')")
    @Log(title = "申请详情验收", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody ProProjectApplyDetailCheck proProjectApplyDetailCheck)
    {
        return toAjax(proProjectApplyDetailCheckService.updateProProjectApplyDetailCheck(proProjectApplyDetailCheck));
    }

    /**
     * 删除申请详情验收
     */
    @PreAuthorize("@ss.hasPermi('manage:proProjectApplyDetailCheck:remove')")
    @Log(title = "申请详情验收", businessType = BusinessType.DELETE)
	@DeleteMapping("/{detailIds}")
    public AjaxResult remove(@PathVariable Long[] detailIds)
    {
        return toAjax(proProjectApplyDetailCheckService.deleteProProjectApplyDetailCheckByDetailIds(detailIds));
    }
}
