package com.yanzhu.web;

import java.util.Date;
import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.yanzhu.common.constant.Constants;
import com.yanzhu.common.core.domain.entity.SysUser;
import com.yanzhu.common.core.redis.RedisCache;
import com.yanzhu.common.core.text.Convert;
import com.yanzhu.common.enums.ShiFouEnum;
import com.yanzhu.common.utils.DateUtils;
import com.yanzhu.common.utils.StringUtils;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.yanzhu.common.annotation.Log;
import com.yanzhu.common.core.controller.BaseController;
import com.yanzhu.common.core.domain.AjaxResult;
import com.yanzhu.common.enums.BusinessType;
import com.yanzhu.project.domain.ProProjectInfo;
import com.yanzhu.project.service.IProProjectInfoService;
import com.yanzhu.common.utils.poi.ExcelUtil;
import com.yanzhu.common.core.page.TableDataInfo;

/**
 * 项目信息Controller
 * 
 * @author yanZhu
 * @date 2024-02-23
 */
@RestController
@RequestMapping("/project/projectInfo")
public class ProProjectInfoController extends BaseController
{

    @Autowired
    private RedisCache redisCache;

    @Autowired
    private IProProjectInfoService proProjectInfoService;

    /**
     * 查询项目信息列表
     */
    @PreAuthorize("@ss.hasPermi('project:projectInfo:list')")
    @GetMapping("/list")
    public TableDataInfo list(ProProjectInfo proProjectInfo)
    {
        startPage();
        List<ProProjectInfo> list = proProjectInfoService.selectProProjectInfoList(proProjectInfo);
        return getDataTable(list);
    }

    /**
     * 导出项目信息列表
     */
    @PreAuthorize("@ss.hasPermi('project:projectInfo:export')")
    @Log(title = "项目信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ProProjectInfo proProjectInfo)
    {
        List<ProProjectInfo> list = proProjectInfoService.selectProProjectInfoList(proProjectInfo);
        ExcelUtil<ProProjectInfo> util = new ExcelUtil<ProProjectInfo>(ProProjectInfo.class);
        util.exportExcel(response, list, "项目信息数据");
    }

    /**
     * 获取项目信息详细信息
     */
    @PreAuthorize("@ss.hasPermi('project:projectInfo:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(proProjectInfoService.selectProProjectInfoById(id));
    }

    /**
     * 新增项目信息
     */
    @PreAuthorize("@ss.hasPermi('project:projectInfo:add')")
    @Log(title = "项目信息", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody ProProjectInfo proProjectInfo)
    {
        int res = proProjectInfoService.insertProProjectInfo(proProjectInfo);
        if(res>0){
            String key = "YANZHU.PROJ.V1.findMyProjectList."+proProjectInfo.getDeptId();
            redisCache.deleteObject(key);
        }
        return toAjax(res);
    }

    /**
     * 修改项目信息
     */
    @PreAuthorize("@ss.hasPermi('project:projectInfo:edit')")
    @Log(title = "项目信息", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody ProProjectInfo proProjectInfo)
    {
        int res = proProjectInfoService.updateProProjectInfo(proProjectInfo);
        if(res>0){
            String key = "YANZHU.PROJ.V1.findMyProjectList."+proProjectInfo.getDeptId();
            redisCache.deleteObject(key);
        }
        return toAjax(res);
    }

    /**
     * 删除项目信息
     */
    @PreAuthorize("@ss.hasPermi('project:projectInfo:remove')")
    @Log(title = "项目信息", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        int res = proProjectInfoService.deleteProProjectInfoByIds(ids);
        if(res>0){
            for(Long id:ids){
                ProProjectInfo proProjectInfo = proProjectInfoService.selectProProjectInfoById(id);
                String key = "YANZHU.PROJ.V1.findMyProjectList."+proProjectInfo.getDeptId();
                redisCache.deleteObject(key);
            }
        }
        return toAjax(res);
    }

    /**
     * 获取临建项目编号
     */
    @GetMapping(value = "/findMyDeptProjectNo")
    public AjaxResult findMyDeptProjectNo()
    {
        Long parDeptId = super.getLoginUser().getUser().getParDeptId();
        ProProjectInfo proProjectInfo = new ProProjectInfo();
        proProjectInfo.setDeptId(parDeptId);
        int count=1;
        List<ProProjectInfo> list = proProjectInfoService.selectProProjectInfoList(proProjectInfo);
        if(StringUtils.isNotEmpty(list)){
            count = count+list.size();
        }
        String proNo = parDeptId+DateUtils.parseDateToStr(DateUtils.YYYY,new Date())+String.format("%03d", count);
        AjaxResult ajax = AjaxResult.success();
        ajax.put("data", proNo);
        return ajax;
    }

    /**
     * 获取项目信息详细信息
     */
    @GetMapping(value = "/findMyDeptProject")
    public AjaxResult findMyDeptProject()
    {
        ProProjectInfo proProjectInfo = new ProProjectInfo();
        //超管查询所有数据
        if(!SysUser.isAdmin(super.getUserId())){
            proProjectInfo.setDeptId(Convert.toLong(super.getLoginUser().getUser().getParDeptId()));
        }
        proProjectInfo.setIsDel(ShiFouEnum.FOU.getCode());
        List<ProProjectInfo> list = proProjectInfoService.selectProProjectInfoList(proProjectInfo);
        return success(list);
    }
}
