package com.yanzhu.project.service.impl;

import com.yanzhu.common.core.text.Convert;
import com.yanzhu.common.utils.DateUtils;
import com.yanzhu.common.utils.SecurityUtils;
import com.yanzhu.project.domain.ProProjectApplyDetail;
import com.yanzhu.project.domain.ProProjectApplyDetailCheck;
import com.yanzhu.project.mapper.ProProjectApplyDetailCheckMapper;
import com.yanzhu.project.mapper.ProProjectApplyMapper;
import com.yanzhu.project.service.IProProjectApplyDetailCheckService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 申请详情验收Service业务层处理
 * 
 * @author yanZhu
 * @date 2024-09-01
 */
@Service
public class ProProjectApplyDetailCheckServiceImpl implements IProProjectApplyDetailCheckService
{
    @Autowired
    private ProProjectApplyMapper proProjectApplyMapper;

    @Autowired
    private ProProjectApplyDetailCheckMapper proProjectApplyDetailCheckMapper;

    /**
     * 查询申请详情验收
     * 
     * @param detailId 申请详情验收主键
     * @return 申请详情验收
     */
    @Override
    public List<ProProjectApplyDetailCheck> selectProProjectApplyDetailCheckByDetailId(Long detailId)
    {
        return proProjectApplyDetailCheckMapper.selectProProjectApplyDetailCheckByDetailId(detailId);
    }

    /**
     * 查询申请详情验收列表
     * 
     * @param proProjectApplyDetailCheck 申请详情验收
     * @return 申请详情验收
     */
    @Override
    public List<ProProjectApplyDetailCheck> selectProProjectApplyDetailCheckList(ProProjectApplyDetailCheck proProjectApplyDetailCheck)
    {
        return proProjectApplyDetailCheckMapper.selectProProjectApplyDetailCheckList(proProjectApplyDetailCheck);
    }

    /**
     * 新增申请详情验收
     * 
     * @param proProjectApplyDetailCheck 申请详情验收
     * @return 结果
     */
    @Override
    public int insertProProjectApplyDetailCheck(ProProjectApplyDetailCheck proProjectApplyDetailCheck)
    {
        proProjectApplyDetailCheck.setCreateBy(SecurityUtils.getLoginUser().getUser().getNickName());
        proProjectApplyDetailCheck.setCreateTime(DateUtils.getNowDate());
        int res = proProjectApplyDetailCheckMapper.insertProProjectApplyDetailCheck(proProjectApplyDetailCheck);
        if(res>0){
            ProProjectApplyDetail proProjectApplyDetail = proProjectApplyMapper.selectProProjectApplyDetailById(proProjectApplyDetailCheck.getDetailId());
            proProjectApplyDetail.setIsChecked(1);
            proProjectApplyDetail.setCheckedTotalValue(Convert.toStr(Convert.toDouble(proProjectApplyDetail.getCheckedTotalValue())+Convert.toDouble(proProjectApplyDetailCheck.getCheckValue())));
            proProjectApplyMapper.updateProProjectApplyDetail(proProjectApplyDetail);
        }
        return res;
    }

    /**
     * 修改申请详情验收
     * 
     * @param proProjectApplyDetailCheck 申请详情验收
     * @return 结果
     */
    @Override
    public int updateProProjectApplyDetailCheck(ProProjectApplyDetailCheck proProjectApplyDetailCheck)
    {
        return proProjectApplyDetailCheckMapper.updateProProjectApplyDetailCheck(proProjectApplyDetailCheck);
    }

    /**
     * 批量删除申请详情验收
     * 
     * @param detailIds 需要删除的申请详情验收主键
     * @return 结果
     */
    @Override
    public int deleteProProjectApplyDetailCheckByDetailIds(Long[] detailIds)
    {
        return proProjectApplyDetailCheckMapper.deleteProProjectApplyDetailCheckByDetailIds(detailIds);
    }

    /**
     * 删除申请详情验收信息
     * 
     * @param detailId 申请详情验收主键
     * @return 结果
     */
    @Override
    public int deleteProProjectApplyDetailCheckByDetailId(Long detailId)
    {
        return proProjectApplyDetailCheckMapper.deleteProProjectApplyDetailCheckByDetailId(detailId);
    }
}
